@@ -244,7 +244,8 @@ def _make_index(params, conn_table):
     ix = sa_schema.Index(
         params["name"],
         *[conn_table.c[cname] for cname in params["column_names"]],
-        unique=params["unique"]
+        unique=params["unique"],
+        _table=conn_table
     )
     if "duplicates_constraint" in params:
         ix.info["duplicates_constraint"] = params["duplicates_constraint"]
@@ -705,9 +706,20 @@ def _compare_indexes_and_uniques(
                     ops.AddConstraintOp.from_constraint(new.const)
                 )
 
-    for added_name in sorted(set(metadata_names).difference(conn_names)):
-        obj = metadata_names[added_name]
-        obj_added(obj)
+    for removed_name in sorted(set(conn_names).difference(metadata_names)):
+        conn_obj = conn_names[removed_name]
+        if not conn_obj.is_index and conn_obj.sig in unnamed_metadata_uniques:
+            continue
+        elif removed_name in doubled_constraints:
+            if (
+                conn_obj.sig not in metadata_indexes_by_sig
+                and conn_obj.sig not in metadata_uniques_by_sig
+            ):
+                conn_uq, conn_idx = doubled_constraints[removed_name]
+                obj_removed(conn_uq)
+                obj_removed(conn_idx)
+        else:
+            obj_removed(conn_obj)
 
     for existing_name in sorted(set(metadata_names).intersection(conn_names)):
         metadata_obj = metadata_names[existing_name]
@@ -739,20 +751,9 @@ def _compare_indexes_and_uniques(
             if msg:
                 obj_changed(conn_obj, metadata_obj, msg)
 
-    for removed_name in sorted(set(conn_names).difference(metadata_names)):
-        conn_obj = conn_names[removed_name]
-        if not conn_obj.is_index and conn_obj.sig in unnamed_metadata_uniques:
-            continue
-        elif removed_name in doubled_constraints:
-            if (
-                conn_obj.sig not in metadata_indexes_by_sig
-                and conn_obj.sig not in metadata_uniques_by_sig
-            ):
-                conn_uq, conn_idx = doubled_constraints[removed_name]
-                obj_removed(conn_uq)
-                obj_removed(conn_idx)
-        else:
-            obj_removed(conn_obj)
+    for added_name in sorted(set(metadata_names).difference(conn_names)):
+        obj = metadata_names[added_name]
+        obj_added(obj)
 
     for uq_sig in unnamed_metadata_uniques:
         if uq_sig not in conn_uniques_by_sig:
@@ -837,17 +838,21 @@ def _compare_nullable(
     metadata_col,
 ):
 
-    # work around SQLAlchemy issue #3023
-    if metadata_col.primary_key:
-        return
-
     metadata_col_nullable = metadata_col.nullable
     conn_col_nullable = conn_col.nullable
     alter_column_op.existing_nullable = conn_col_nullable
 
     if conn_col_nullable is not metadata_col_nullable:
-        if sqla_compat._server_default_is_identity(
-            metadata_col.server_default, conn_col.server_default
+        if (
+            sqla_compat._server_default_is_computed(
+                metadata_col.server_default, conn_col.server_default
+            )
+            and sqla_compat._nullability_might_be_unset(metadata_col)
+            or (
+                sqla_compat._server_default_is_identity(
+                    metadata_col.server_default, conn_col.server_default
+                )
+            )
         ):
             log.info(
                 "Ignoring nullable change on identity column '%s.%s'",
@@ -1024,11 +1029,11 @@ def _compare_identity_default(
     metadata_col,
 ):
     impl = autogen_context.migration_context.impl
-    diff, ignored_attr = impl._compare_identity_default(
+    diff, ignored_attr, is_alter = impl._compare_identity_default(
         metadata_col.server_default, conn_col.server_default
     )
 
-    return diff
+    return diff, is_alter
 
 
 @comparators.dispatch_for("column")
@@ -1075,7 +1080,7 @@ def _compare_server_default(
         metadata_default, conn_col_default
     ):
         alter_column_op.existing_server_default = conn_col_default
-        is_diff = _compare_identity_default(
+        diff, is_alter = _compare_identity_default(
             autogen_context,
             alter_column_op,
             schema,
@@ -1084,15 +1089,15 @@ def _compare_server_default(
             conn_col,
             metadata_col,
         )
-        if is_diff or (bool(conn_col_default) != bool(metadata_default)):
+        if is_alter:
             alter_column_op.modify_server_default = metadata_default
-            if is_diff:
+            if diff:
                 log.info(
                     "Detected server default on column '%s.%s': "
                     "identity options attributes %s",
                     tname,
                     cname,
-                    sorted(is_diff),
+                    sorted(diff),
                 )
     else:
         rendered_metadata_default = _render_server_default_for_compare(
