@@ -1,6 +1,6 @@
-from ..operations import Operations
 from .migration import MigrationContext
 from .. import util
+from ..operations import Operations
 
 
 class EnvironmentContext(util.ModuleClsProxy):
@@ -120,7 +120,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         has been configured.
 
         """
-        return self.context_opts.get('as_sql', False)
+        return self.context_opts.get("as_sql", False)
 
     def is_transactional_ddl(self):
         """Return True if the context is configured to expect a
@@ -163,8 +163,6 @@ class EnvironmentContext(util.ModuleClsProxy):
         This function does not require that the :class:`.MigrationContext`
         has been configured.
 
-        .. versionadded:: 0.7.0
-
         """
         return self.script.as_revision_number("heads")
 
@@ -182,17 +180,20 @@ class EnvironmentContext(util.ModuleClsProxy):
         """
         if self._migration_context is not None:
             return self.script.as_revision_number(
-                self.get_context()._start_from_rev)
-        elif 'starting_rev' in self.context_opts:
+                self.get_context()._start_from_rev
+            )
+        elif "starting_rev" in self.context_opts:
             return self.script.as_revision_number(
-                self.context_opts['starting_rev'])
+                self.context_opts["starting_rev"]
+            )
         else:
             # this should raise only in the case that a command
             # is being run where the "starting rev" is never applicable;
             # this is to catch scripts which rely upon this in
             # non-sql mode or similar
             raise util.CommandError(
-                "No starting revision argument is available.")
+                "No starting revision argument is available."
+            )
 
     def get_revision_argument(self):
         """Get the 'destination' revision argument.
@@ -209,7 +210,8 @@ class EnvironmentContext(util.ModuleClsProxy):
 
         """
         return self.script.as_revision_number(
-            self.context_opts['destination_rev'])
+            self.context_opts["destination_rev"]
+        )
 
     def get_tag_argument(self):
         """Return the value passed for the ``--tag`` argument, if any.
@@ -229,7 +231,7 @@ class EnvironmentContext(util.ModuleClsProxy):
             line.
 
         """
-        return self.context_opts.get('tag', None)
+        return self.context_opts.get("tag", None)
 
     def get_x_argument(self, as_dictionary=False):
         """Return the value(s) passed for the ``-x`` argument, if any.
@@ -263,8 +265,6 @@ class EnvironmentContext(util.ModuleClsProxy):
         This function does not require that the :class:`.MigrationContext`
         has been configured.
 
-        .. versionadded:: 0.6.0
-
         .. seealso::
 
             :meth:`.EnvironmentContext.get_tag_argument`
@@ -277,39 +277,39 @@ class EnvironmentContext(util.ModuleClsProxy):
         else:
             value = []
         if as_dictionary:
-            value = dict(
-                arg.split('=', 1) for arg in value
-            )
+            value = dict(arg.split("=", 1) for arg in value)
         return value
 
-    def configure(self,
-                  connection=None,
-                  url=None,
-                  dialect_name=None,
-                  transactional_ddl=None,
-                  transaction_per_migration=False,
-                  output_buffer=None,
-                  starting_rev=None,
-                  tag=None,
-                  template_args=None,
-                  render_as_batch=False,
-                  target_metadata=None,
-                  include_symbol=None,
-                  include_object=None,
-                  include_schemas=False,
-                  process_revision_directives=None,
-                  compare_type=False,
-                  compare_server_default=False,
-                  render_item=None,
-                  literal_binds=False,
-                  upgrade_token="upgrades",
-                  downgrade_token="downgrades",
-                  alembic_module_prefix="op.",
-                  sqlalchemy_module_prefix="sa.",
-                  user_module_prefix=None,
-                  on_version_apply=None,
-                  **kw
-                  ):
+    def configure(
+        self,
+        connection=None,
+        url=None,
+        dialect_name=None,
+        dialect_opts=None,
+        transactional_ddl=None,
+        transaction_per_migration=False,
+        output_buffer=None,
+        starting_rev=None,
+        tag=None,
+        template_args=None,
+        render_as_batch=False,
+        target_metadata=None,
+        include_name=None,
+        include_object=None,
+        include_schemas=False,
+        process_revision_directives=None,
+        compare_type=False,
+        compare_server_default=False,
+        render_item=None,
+        literal_binds=False,
+        upgrade_token="upgrades",
+        downgrade_token="downgrades",
+        alembic_module_prefix="op.",
+        sqlalchemy_module_prefix="sa.",
+        user_module_prefix=None,
+        on_version_apply=None,
+        **kw
+    ):
         """Configure a :class:`.MigrationContext` within this
         :class:`.EnvironmentContext` which will provide database
         connectivity and other configuration to a series of
@@ -352,6 +352,11 @@ class EnvironmentContext(util.ModuleClsProxy):
          "postgresql", "mssql", etc.
          The type of dialect to be used will be derived from this if
          ``connection`` and ``url`` are not passed.
+        :param dialect_opts: dictionary of options to be passed to dialect
+         constructor.
+
+         .. versionadded:: 1.0.12
+
         :param transactional_ddl: Force the usage of "transactional"
          DDL on or off;
          this otherwise defaults to whether or not the dialect in
@@ -359,9 +364,6 @@ class EnvironmentContext(util.ModuleClsProxy):
         :param transaction_per_migration: if True, nest each migration script
          in a transaction rather than the full series of migrations to
          run.
-
-         .. versionadded:: 0.6.5
-
         :param output_buffer: a file-like object that will be used
          for textual output
          when the ``--sql`` option is used to generate SQL scripts.
@@ -385,8 +387,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          .. note:: the ``literal_binds`` flag is ignored on SQLAlchemy
             versions prior to 0.8 where this feature is not supported.
 
-         .. versionadded:: 0.7.6
-
          .. seealso::
 
             :meth:`.Operations.inline_literal`
@@ -411,12 +411,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          only takes effect when the table is first created.
          Defaults to True; setting to False should not be necessary and is
          here for backwards compatibility reasons.
-
-         .. versionadded:: 0.8.10  Added the
-            :paramref:`.EnvironmentContext.configure.version_table_pk`
-            flag and additionally established that the Alembic version table
-            has a primary key constraint by default.
-
         :param on_version_apply: a callable or collection of callables to be
             run for each migration step.
             The callables will be run in the order they are given, once for
@@ -432,9 +426,6 @@ class EnvironmentContext(util.ModuleClsProxy):
               current heads,
             * ``run_args``: the ``**kwargs`` passed to :meth:`.run_migrations`.
 
-            .. versionadded:: 0.9.3
-
-
         Parameters specific to the autogenerate feature, when
         ``alembic revision`` is run with the ``--autogenerate`` feature:
 
@@ -446,14 +437,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          what is locally available on the target
          :class:`~sqlalchemy.engine.Connection`
          to produce candidate upgrade/downgrade operations.
-
-         .. versionchanged:: 0.9.0 the
-            :paramref:`.EnvironmentContext.configure.target_metadata`
-            parameter may now be passed a sequence of
-            :class:`~sqlalchemy.schema.MetaData` objects to support
-            autogeneration of multiple :class:`~sqlalchemy.schema.MetaData`
-            collections.
-
         :param compare_type: Indicates type comparison behavior during
          an autogenerate
          operation.  Defaults to ``False`` which disables type
@@ -512,6 +495,49 @@ class EnvironmentContext(util.ModuleClsProxy):
 
             :paramref:`.EnvironmentContext.configure.compare_type`
 
+        :param include_name: A callable function which is given
+         the chance to return ``True`` or ``False`` for any database reflected
+         object based on its name, including database schema names when
+         the :paramref:`.EnvironmentContext.configure.include_schemas` flag
+         is set to ``True``.
+
+         The function accepts the following positional arguments:
+
+         * ``name``: the name of the object, such as schema name or table name.
+           Will be ``None`` when indicating the default schema name of the
+           database connection.
+         * ``type``: a string describing the type of object; currently
+           ``"schema"``, ``"table"``, ``"column"``, ``"index"``,
+           ``"unique_constraint"``, or ``"foreign_key_constraint"``
+         * ``parent_names``: a dictionary of "parent" object names, that are
+           relative to the name being given.  Keys in this dictionary may
+           include:  ``"schema_name"``, ``"table_name"``.
+
+         E.g.::
+
+            def include_name(name, type_, parent_names):
+                if type_ == "schema":
+                    return name in ["schema_one", "schema_two"]
+                else:
+                    return True
+
+            context.configure(
+                # ...
+                include_schemas = True,
+                include_name = include_name
+            )
+
+         .. versionadded:: 1.5
+
+         .. seealso::
+
+            :ref:`autogenerate_include_hooks`
+
+            :paramref:`.EnvironmentContext.configure.include_object`
+
+            :paramref:`.EnvironmentContext.configure.include_schemas`
+
+
         :param include_object: A callable function which is given
          the chance to return ``True`` or ``False`` for any object,
          indicating if the given object should be considered in the
@@ -530,14 +556,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          * ``type``: a string describing the type of object; currently
            ``"table"``, ``"column"``, ``"index"``, ``"unique_constraint"``,
            or ``"foreign_key_constraint"``
-
-           .. versionadded:: 0.7.0 Support for indexes and unique constraints
-              within the
-              :paramref:`~.EnvironmentContext.configure.include_object` hook.
-
-           .. versionadded:: 0.7.1 Support for foreign keys within the
-              :paramref:`~.EnvironmentContext.configure.include_object` hook.
-
          * ``reflected``: ``True`` if the given object was produced based on
            table reflection, ``False`` if it's from a local :class:`.MetaData`
            object.
@@ -559,52 +577,27 @@ class EnvironmentContext(util.ModuleClsProxy):
                 include_object = include_object
             )
 
-         :paramref:`.EnvironmentContext.configure.include_object` can also
-         be used to filter on specific schemas to include or omit, when
-         the :paramref:`.EnvironmentContext.configure.include_schemas`
-         flag is set to ``True``.   The :attr:`.Table.schema` attribute
-         on each :class:`.Table` object reflected will indicate the name of the
-         schema from which the :class:`.Table` originates.
-
-         .. versionadded:: 0.6.0
+         For the use case of omitting specific schemas from a target database
+         when :paramref:`.EnvironmentContext.configure.include_schemas` is
+         set to ``True``, the :attr:`~sqlalchemy.schema.Table.schema`
+         attribute can be checked for each :class:`~sqlalchemy.schema.Table`
+         object passed to the hook, however it is much more efficient
+         to filter on schemas before reflection of objects takes place
+         using the :paramref:`.EnvironmentContext.configure.include_name`
+         hook.
 
          .. seealso::
 
-            :paramref:`.EnvironmentContext.configure.include_schemas`
+            :ref:`autogenerate_include_hooks`
 
-        :param include_symbol: A callable function which, given a table name
-         and schema name (may be ``None``), returns ``True`` or ``False``,
-         indicating if the given table should be considered in the
-         autogenerate sweep.
-
-         .. deprecated:: 0.6.0
-            :paramref:`.EnvironmentContext.configure.include_symbol`
-            is superceded by the more generic
-            :paramref:`.EnvironmentContext.configure.include_object`
-            parameter.
-
-         E.g.::
-
-            def include_symbol(tablename, schema):
-                return tablename not in ("skip_table_one", "skip_table_two")
-
-            context.configure(
-                # ...
-                include_symbol = include_symbol
-            )
-
-         .. seealso::
+            :paramref:`.EnvironmentContext.configure.include_name`
 
             :paramref:`.EnvironmentContext.configure.include_schemas`
 
-            :paramref:`.EnvironmentContext.configure.include_object`
-
         :param render_as_batch: if True, commands which alter elements
          within a table will be placed under a ``with batch_alter_table():``
          directive, so that batch migrations will take place.
 
-         .. versionadded:: 0.7.0
-
          .. seealso::
 
             :ref:`batch_migrations`
@@ -614,12 +607,16 @@ class EnvironmentContext(util.ModuleClsProxy):
          :meth:`~sqlalchemy.engine.reflection.Inspector.get_schema_names`
          method, and include all differences in tables found across all
          those schemas.  When using this option, you may want to also
-         use the :paramref:`.EnvironmentContext.configure.include_object`
-         option to specify a callable which
+         use the :paramref:`.EnvironmentContext.configure.include_name`
+         parameter to specify a callable which
          can filter the tables/schemas that get included.
 
          .. seealso::
 
+            :ref:`autogenerate_include_hooks`
+
+            :paramref:`.EnvironmentContext.configure.include_name`
+
             :paramref:`.EnvironmentContext.configure.include_object`
 
         :param render_item: Callable that can be used to override how
@@ -685,15 +682,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          in order to future-proof migration files against reorganizations
          in modules.
 
-         .. versionchanged:: 0.7.0
-            :paramref:`.EnvironmentContext.configure.user_module_prefix`
-            no longer defaults to the value of
-            :paramref:`.EnvironmentContext.configure.sqlalchemy_module_prefix`
-            when left at ``None``; the ``__module__`` attribute is now used.
-
-         .. versionadded:: 0.6.3 added
-            :paramref:`.EnvironmentContext.configure.user_module_prefix`
-
          .. seealso::
 
             :ref:`autogen_module_prefix`
@@ -731,18 +719,6 @@ class EnvironmentContext(util.ModuleClsProxy):
          a :class:`.Rewriter` object.  This is a helper object that
          assists in the production of autogenerate-stream rewriter functions.
 
-
-         .. versionadded:: 0.8.0
-
-         .. versionchanged:: 0.8.1 - The
-            :paramref:`.EnvironmentContext.configure.process_revision_directives`
-            hook can append op directives into :class:`.UpgradeOps` and
-            :class:`.DowngradeOps` which will be rendered in Python regardless
-            of whether the ``--autogenerate`` option is in use or not;
-            the ``revision_environment`` configuration variable should be
-            set to "true" in the config to enable this.
-
-
          .. seealso::
 
              :ref:`customizing_revision`
@@ -774,33 +750,33 @@ class EnvironmentContext(util.ModuleClsProxy):
         elif self.config.output_buffer is not None:
             opts["output_buffer"] = self.config.output_buffer
         if starting_rev:
-            opts['starting_rev'] = starting_rev
+            opts["starting_rev"] = starting_rev
         if tag:
-            opts['tag'] = tag
-        if template_args and 'template_args' in opts:
-            opts['template_args'].update(template_args)
+            opts["tag"] = tag
+        if template_args and "template_args" in opts:
+            opts["template_args"].update(template_args)
         opts["transaction_per_migration"] = transaction_per_migration
-        opts['target_metadata'] = target_metadata
-        opts['include_symbol'] = include_symbol
-        opts['include_object'] = include_object
-        opts['include_schemas'] = include_schemas
-        opts['render_as_batch'] = render_as_batch
-        opts['upgrade_token'] = upgrade_token
-        opts['downgrade_token'] = downgrade_token
-        opts['sqlalchemy_module_prefix'] = sqlalchemy_module_prefix
-        opts['alembic_module_prefix'] = alembic_module_prefix
-        opts['user_module_prefix'] = user_module_prefix
-        opts['literal_binds'] = literal_binds
-        opts['process_revision_directives'] = process_revision_directives
-        opts['on_version_apply'] = util.to_tuple(on_version_apply, default=())
+        opts["target_metadata"] = target_metadata
+        opts["include_name"] = include_name
+        opts["include_object"] = include_object
+        opts["include_schemas"] = include_schemas
+        opts["render_as_batch"] = render_as_batch
+        opts["upgrade_token"] = upgrade_token
+        opts["downgrade_token"] = downgrade_token
+        opts["sqlalchemy_module_prefix"] = sqlalchemy_module_prefix
+        opts["alembic_module_prefix"] = alembic_module_prefix
+        opts["user_module_prefix"] = user_module_prefix
+        opts["literal_binds"] = literal_binds
+        opts["process_revision_directives"] = process_revision_directives
+        opts["on_version_apply"] = util.to_tuple(on_version_apply, default=())
 
         if render_item is not None:
-            opts['render_item'] = render_item
+            opts["render_item"] = render_item
         if compare_type is not None:
-            opts['compare_type'] = compare_type
+            opts["compare_type"] = compare_type
         if compare_server_default is not None:
-            opts['compare_server_default'] = compare_server_default
-        opts['script'] = self.script
+            opts["compare_server_default"] = compare_server_default
+        opts["script"] = self.script
 
         opts.update(kw)
 
@@ -809,7 +785,8 @@ class EnvironmentContext(util.ModuleClsProxy):
             url=url,
             dialect_name=dialect_name,
             environment_context=self,
-            opts=opts
+            dialect_opts=dialect_opts,
+            opts=opts,
         )
 
     def run_migrations(self, **kw):
@@ -847,8 +824,7 @@ class EnvironmentContext(util.ModuleClsProxy):
         first been made available via :meth:`.configure`.
 
         """
-        self.get_context().execute(sql,
-                                   execution_options=execution_options)
+        self.get_context().execute(sql, execution_options=execution_options)
 
     def static_output(self, text):
         """Emit text directly to the "offline" SQL stream.
