@@ -1,21 +1,24 @@
+from contextlib import contextmanager
 import datetime
 import os
 import re
 import shutil
-from .. import util
-from ..util import compat
+
+from dateutil import tz
+
 from . import revision
+from . import write_hooks
+from .. import util
 from ..runtime import migration
+from ..util import compat
 
-from contextlib import contextmanager
-
-_sourceless_rev_file = re.compile(r'(?!\.\#|__init__)(.*\.py)(c|o)?$')
-_only_source_rev_file = re.compile(r'(?!\.\#|__init__)(.*\.py)$')
-_legacy_rev = re.compile(r'([a-f0-9]+)\.py$')
-_mod_def_re = re.compile(r'(upgrade|downgrade)_([a-z0-9]+)')
-_slug_re = re.compile(r'\w+')
+_sourceless_rev_file = re.compile(r"(?!\.\#|__init__)(.*\.py)(c|o)?$")
+_only_source_rev_file = re.compile(r"(?!\.\#|__init__)(.*\.py)$")
+_legacy_rev = re.compile(r"([a-f0-9]+)\.py$")
+_mod_def_re = re.compile(r"(upgrade|downgrade)_([a-z0-9]+)")
+_slug_re = re.compile(r"\w+")
 _default_file_template = "%(rev)s_%(slug)s"
-_split_on_space_comma = re.compile(r',|(?: +)')
+_split_on_space_comma = re.compile(r", *|(?: +)")
 
 
 class ScriptDirectory(object):
@@ -39,10 +42,17 @@ class ScriptDirectory(object):
 
     """
 
-    def __init__(self, dir, file_template=_default_file_template,
-                 truncate_slug_length=40,
-                 version_locations=None,
-                 sourceless=False, output_encoding="utf-8"):
+    def __init__(
+        self,
+        dir,  # noqa
+        file_template=_default_file_template,
+        truncate_slug_length=40,
+        version_locations=None,
+        sourceless=False,
+        output_encoding="utf-8",
+        timezone=None,
+        hook_config=None,
+    ):
         self.dir = dir
         self.file_template = file_template
         self.version_locations = version_locations
@@ -50,11 +60,15 @@ class ScriptDirectory(object):
         self.sourceless = sourceless
         self.output_encoding = output_encoding
         self.revision_map = revision.RevisionMap(self._load_revisions)
+        self.timezone = timezone
+        self.hook_config = hook_config
 
         if not os.access(dir, os.F_OK):
-            raise util.CommandError("Path doesn't exist: %r.  Please use "
-                                    "the 'init' command to create a new "
-                                    "scripts folder." % dir)
+            raise util.CommandError(
+                "Path doesn't exist: %r.  Please use "
+                "the 'init' command to create a new "
+                "scripts folder." % os.path.abspath(dir)
+            )
 
     @property
     def versions(self):
@@ -72,19 +86,21 @@ class ScriptDirectory(object):
                 for location in self.version_locations
             ]
         else:
-            return (os.path.abspath(os.path.join(self.dir, 'versions')),)
+            return (os.path.abspath(os.path.join(self.dir, "versions")),)
 
     def _load_revisions(self):
         if self.version_locations:
             paths = [
-                vers for vers in self._version_locations
-                if os.path.exists(vers)]
+                vers
+                for vers in self._version_locations
+                if os.path.exists(vers)
+            ]
         else:
             paths = [self.versions]
 
         dupes = set()
         for vers in paths:
-            for file_ in os.listdir(vers):
+            for file_ in Script._list_py_dir(self, vers):
                 path = os.path.realpath(os.path.join(vers, file_))
                 if path in dupes:
                     util.warn(
@@ -107,10 +123,11 @@ class ScriptDirectory(object):
         present.
 
         """
-        script_location = config.get_main_option('script_location')
+        script_location = config.get_main_option("script_location")
         if script_location is None:
-            raise util.CommandError("No 'script_location' key "
-                                    "found in configuration.")
+            raise util.CommandError(
+                "No 'script_location' key " "found in configuration."
+            )
         truncate_slug_length = config.get_main_option("truncate_slug_length")
         if truncate_slug_length is not None:
             truncate_slug_length = int(truncate_slug_length)
@@ -118,22 +135,29 @@ class ScriptDirectory(object):
         version_locations = config.get_main_option("version_locations")
         if version_locations:
             version_locations = _split_on_space_comma.split(version_locations)
+
         return ScriptDirectory(
             util.coerce_resource_to_filename(script_location),
             file_template=config.get_main_option(
-                'file_template',
-                _default_file_template),
+                "file_template", _default_file_template
+            ),
             truncate_slug_length=truncate_slug_length,
             sourceless=config.get_main_option("sourceless") == "true",
             output_encoding=config.get_main_option("output_encoding", "utf-8"),
-            version_locations=version_locations
+            version_locations=version_locations,
+            timezone=config.get_main_option("timezone"),
+            hook_config=config.get_section("post_write_hooks", {}),
         )
 
     @contextmanager
     def _catch_revision_errors(
-            self,
-            ancestor=None, multiple_heads=None, start=None, end=None,
-            resolution=None):
+        self,
+        ancestor=None,
+        multiple_heads=None,
+        start=None,
+        end=None,
+        resolution=None,
+    ):
         try:
             yield
         except revision.RangeNotAncestorError as rna:
@@ -147,7 +171,7 @@ class ScriptDirectory(object):
                     "ancestor/descendant revisions along the same branch"
                 )
             ancestor = ancestor % {"start": start, "end": end}
-            compat.raise_from_cause(util.CommandError(ancestor))
+            compat.raise_(util.CommandError(ancestor), from_=rna)
         except revision.MultipleHeads as mh:
             if not multiple_heads:
                 multiple_heads = (
@@ -155,20 +179,21 @@ class ScriptDirectory(object):
                     "argument '%(head_arg)s'; please "
                     "specify a specific target revision, "
                     "'<branchname>@%(head_arg)s' to "
-                    "narrow to a specific head, or 'heads' for all heads")
+                    "narrow to a specific head, or 'heads' for all heads"
+                )
             multiple_heads = multiple_heads % {
                 "head_arg": end or mh.argument,
-                "heads": util.format_as_comma(mh.heads)
+                "heads": util.format_as_comma(mh.heads),
             }
-            compat.raise_from_cause(util.CommandError(multiple_heads))
+            compat.raise_(util.CommandError(multiple_heads), from_=mh)
         except revision.ResolutionError as re:
             if resolution is None:
                 resolution = "Can't locate revision identified by '%s'" % (
                     re.argument
                 )
-            compat.raise_from_cause(util.CommandError(resolution))
+            compat.raise_(util.CommandError(resolution), from_=re)
         except revision.RevisionError as err:
-            compat.raise_from_cause(util.CommandError(err.args[0]))
+            compat.raise_(util.CommandError(err.args[0]), from_=err)
 
     def walk_revisions(self, base="base", head="heads"):
         """Iterate through all revisions.
@@ -180,22 +205,17 @@ class ScriptDirectory(object):
          all head revisions.  May also be "head" to indicate a single
          head revision.
 
-         .. versionchanged:: 0.7.0 the "head" identifier now refers to
-            the head of a non-branched repository only; use "heads" to
-            refer to the set of all head branches simultaneously.
-
         """
         with self._catch_revision_errors(start=base, end=head):
             for rev in self.revision_map.iterate_revisions(
-                    head, base, inclusive=True, assert_relative_length=False):
+                head, base, inclusive=True, assert_relative_length=False
+            ):
                 yield rev
 
     def get_revisions(self, id_):
         """Return the :class:`.Script` instance with the given rev identifier,
         symbolic name, or sequence of identifiers.
 
-        .. versionadded:: 0.7.0
-
         """
         with self._catch_revision_errors():
             return self.revision_map.get_revisions(id_)
@@ -205,7 +225,8 @@ class ScriptDirectory(object):
             top_revs = set(self.revision_map.get_revisions(id_))
             top_revs.update(
                 self.revision_map._get_ancestor_nodes(
-                    list(top_revs), include_dependencies=True)
+                    list(top_revs), include_dependencies=True
+                )
             )
             top_revs = self.revision_map._filter_into_branch_heads(top_revs)
             return top_revs
@@ -232,6 +253,8 @@ class ScriptDirectory(object):
         if not rev:
             # convert () to None
             return None
+        elif id_ == "heads":
+            return rev
         else:
             return rev[0]
 
@@ -268,11 +291,13 @@ class ScriptDirectory(object):
             :meth:`.ScriptDirectory.get_heads`
 
         """
-        with self._catch_revision_errors(multiple_heads=(
-                'The script directory has multiple heads (due to branching).'
-                'Please use get_heads(), or merge the branches using '
-                'alembic merge.'
-        )):
+        with self._catch_revision_errors(
+            multiple_heads=(
+                "The script directory has multiple heads (due to branching)."
+                "Please use get_heads(), or merge the branches using "
+                "alembic merge."
+            )
+        ):
             return self.revision_map.get_current_head()
 
     def get_heads(self):
@@ -303,7 +328,8 @@ class ScriptDirectory(object):
         if len(bases) > 1:
             raise util.CommandError(
                 "The script directory has multiple bases. "
-                "Please use get_bases().")
+                "Please use get_bases()."
+            )
         elif bases:
             return bases[0]
         else:
@@ -315,59 +341,84 @@ class ScriptDirectory(object):
         This is the revision number of all scripts that
         have a ``down_revision`` of None.
 
-        .. versionadded:: 0.7.0
-
         """
         return list(self.revision_map.bases)
 
     def _upgrade_revs(self, destination, current_rev):
         with self._catch_revision_errors(
-                ancestor="Destination %(end)s is not a valid upgrade "
-                "target from current head(s)", end=destination):
+            ancestor="Destination %(end)s is not a valid upgrade "
+            "target from current head(s)",
+            end=destination,
+        ):
             revs = self.revision_map.iterate_revisions(
-                destination, current_rev, implicit_base=True)
+                destination, current_rev, implicit_base=True
+            )
             revs = list(revs)
             return [
                 migration.MigrationStep.upgrade_from_script(
-                    self.revision_map, script)
+                    self.revision_map, script
+                )
                 for script in reversed(list(revs))
             ]
 
     def _downgrade_revs(self, destination, current_rev):
         with self._catch_revision_errors(
-                ancestor="Destination %(end)s is not a valid downgrade "
-                "target from current head(s)", end=destination):
+            ancestor="Destination %(end)s is not a valid downgrade "
+            "target from current head(s)",
+            end=destination,
+        ):
             revs = self.revision_map.iterate_revisions(
-                current_rev, destination, select_for_downgrade=True)
+                current_rev, destination, select_for_downgrade=True
+            )
             return [
                 migration.MigrationStep.downgrade_from_script(
-                    self.revision_map, script)
+                    self.revision_map, script
+                )
                 for script in revs
             ]
 
     def _stamp_revs(self, revision, heads):
         with self._catch_revision_errors(
-                multiple_heads="Multiple heads are present; please specify a "
-                "single target revision"):
+            multiple_heads="Multiple heads are present; please specify a "
+            "single target revision"
+        ):
 
             heads = self.get_revisions(heads)
 
-            # filter for lineage will resolve things like
-            # branchname@base, version@base, etc.
-            filtered_heads = self.revision_map.filter_for_lineage(
-                heads, revision, include_dependencies=True)
-
             steps = []
 
+            if not revision:
+                revision = "base"
+
+            filtered_heads = []
+            for rev in util.to_tuple(revision):
+                if rev:
+                    filtered_heads.extend(
+                        self.revision_map.filter_for_lineage(
+                            heads, rev, include_dependencies=True
+                        )
+                    )
+            filtered_heads = util.unique_list(filtered_heads)
+
             dests = self.get_revisions(revision) or [None]
+
             for dest in dests:
+
                 if dest is None:
                     # dest is 'base'.  Return a "delete branch" migration
                     # for all applicable heads.
-                    steps.extend([
-                        migration.StampStep(head.revision, None, False, True)
-                        for head in filtered_heads
-                    ])
+                    steps.extend(
+                        [
+                            migration.StampStep(
+                                head.revision,
+                                None,
+                                False,
+                                True,
+                                self.revision_map,
+                            )
+                            for head in filtered_heads
+                        ]
+                    )
                     continue
                 elif dest in filtered_heads:
                     # the dest is already in the version table, do nothing.
@@ -376,7 +427,8 @@ class ScriptDirectory(object):
                 # figure out if the dest is a descendant or an
                 # ancestor of the selected nodes
                 descendants = set(
-                    self.revision_map._get_descendant_nodes([dest]))
+                    self.revision_map._get_descendant_nodes([dest])
+                )
                 ancestors = set(self.revision_map._get_ancestor_nodes([dest]))
 
                 if descendants.intersection(filtered_heads):
@@ -385,7 +437,12 @@ class ScriptDirectory(object):
                     assert not ancestors.intersection(filtered_heads)
                     todo_heads = [head.revision for head in filtered_heads]
                     step = migration.StampStep(
-                        todo_heads, dest.revision, False, False)
+                        todo_heads,
+                        dest.revision,
+                        False,
+                        False,
+                        self.revision_map,
+                    )
                     steps.append(step)
                     continue
                 elif ancestors.intersection(filtered_heads):
@@ -393,15 +450,23 @@ class ScriptDirectory(object):
                     # we can treat them as a "merge", single step.
                     todo_heads = [head.revision for head in filtered_heads]
                     step = migration.StampStep(
-                        todo_heads, dest.revision, True, False)
+                        todo_heads,
+                        dest.revision,
+                        True,
+                        False,
+                        self.revision_map,
+                    )
                     steps.append(step)
                     continue
                 else:
                     # destination is in a branch not represented,
                     # treat it as new branch
-                    step = migration.StampStep((), dest.revision, True, True)
+                    step = migration.StampStep(
+                        (), dest.revision, True, True, self.revision_map
+                    )
                     steps.append(step)
                     continue
+
             return steps
 
     def run_env(self):
@@ -413,37 +478,64 @@ class ScriptDirectory(object):
 
 
         """
-        util.load_python_file(self.dir, 'env.py')
+        util.load_python_file(self.dir, "env.py")
 
     @property
     def env_py_location(self):
         return os.path.abspath(os.path.join(self.dir, "env.py"))
 
     def _generate_template(self, src, dest, **kw):
-        util.status("Generating %s" % os.path.abspath(dest),
-                    util.template_to_file,
-                    src,
-                    dest,
-                    self.output_encoding,
-                    **kw
-                    )
+        util.status(
+            "Generating %s" % os.path.abspath(dest),
+            util.template_to_file,
+            src,
+            dest,
+            self.output_encoding,
+            **kw
+        )
 
     def _copy_file(self, src, dest):
-        util.status("Generating %s" % os.path.abspath(dest),
-                    shutil.copy,
-                    src, dest)
+        util.status(
+            "Generating %s" % os.path.abspath(dest), shutil.copy, src, dest
+        )
 
     def _ensure_directory(self, path):
         path = os.path.abspath(path)
         if not os.path.exists(path):
-            util.status(
-                "Creating directory %s" % path,
-                os.makedirs, path)
+            util.status("Creating directory %s" % path, os.makedirs, path)
+
+    def _generate_create_date(self):
+        if self.timezone is not None:
+            # First, assume correct capitalization
+            tzinfo = tz.gettz(self.timezone)
+            if tzinfo is None:
+                # Fall back to uppercase
+                tzinfo = tz.gettz(self.timezone.upper())
+            if tzinfo is None:
+                raise util.CommandError(
+                    "Can't locate timezone: %s" % self.timezone
+                )
+            create_date = (
+                datetime.datetime.utcnow()
+                .replace(tzinfo=tz.tzutc())
+                .astimezone(tzinfo)
+            )
+        else:
+            create_date = datetime.datetime.now()
+        return create_date
 
     def generate_revision(
-            self, revid, message, head=None,
-            refresh=False, splice=False, branch_labels=None,
-            version_path=None, depends_on=None, **kw):
+        self,
+        revid,
+        message,
+        head=None,
+        refresh=False,
+        splice=False,
+        branch_labels=None,
+        version_path=None,
+        depends_on=None,
+        **kw
+    ):
         """Generate a new revision file.
 
         This runs the ``script.py.mako`` template, given
@@ -456,9 +548,6 @@ class ScriptDirectory(object):
         :param head: the head revision to generate against.  Defaults
          to the current "head" if no branches are present, else raises
          an exception.
-
-         .. versionadded:: 0.7.0
-
         :param splice: if True, allow the "head" version to not be an
          actual head; otherwise, the selected head must be a head
          (e.g. endpoint) revision.
@@ -468,17 +557,24 @@ class ScriptDirectory(object):
         if head is None:
             head = "head"
 
-        with self._catch_revision_errors(multiple_heads=(
-            "Multiple heads are present; please specify the head "
-            "revision on which the new revision should be based, "
-            "or perform a merge."
-        )):
+        try:
+            Script.verify_rev_id(revid)
+        except revision.RevisionError as err:
+            compat.raise_(util.CommandError(err.args[0]), from_=err)
+
+        with self._catch_revision_errors(
+            multiple_heads=(
+                "Multiple heads are present; please specify the head "
+                "revision on which the new revision should be based, "
+                "or perform a merge."
+            )
+        ):
             heads = self.revision_map.get_revisions(head)
 
         if len(set(heads)) != len(heads):
             raise util.CommandError("Duplicate head revisions specified")
 
-        create_date = datetime.datetime.now()
+        create_date = self._generate_create_date()
 
         if version_path is None:
             if len(self._version_locations) > 1:
@@ -489,7 +585,8 @@ class ScriptDirectory(object):
                 else:
                     raise util.CommandError(
                         "Multiple version locations present, "
-                        "please specify --version-path")
+                        "please specify --version-path"
+                    )
             else:
                 version_path = self.versions
 
@@ -500,7 +597,8 @@ class ScriptDirectory(object):
         else:
             raise util.CommandError(
                 "Path %s is not represented in current "
-                "version locations" % version_path)
+                "version locations" % version_path
+            )
 
         if self.version_locations:
             self._ensure_directory(version_path)
@@ -513,7 +611,8 @@ class ScriptDirectory(object):
                     raise util.CommandError(
                         "Revision %s is not a head revision; please specify "
                         "--splice to create a new branch from this revision"
-                        % head.revision)
+                        % head.revision
+                    )
 
         if depends_on:
             with self._catch_revision_errors():
@@ -525,7 +624,6 @@ class ScriptDirectory(object):
                         (self.revision_map.get_revision(dep), dep)
                         for dep in util.to_list(depends_on)
                     ]
-
                 ]
 
         self._generate_template(
@@ -533,7 +631,8 @@ class ScriptDirectory(object):
             path,
             up_revision=str(revid),
             down_revision=revision.tuple_rev_as_scalar(
-                tuple(h.revision if h is not None else None for h in heads)),
+                tuple(h.revision if h is not None else None for h in heads)
+            ),
             branch_labels=util.to_tuple(branch_labels),
             depends_on=revision.tuple_rev_as_scalar(depends_on),
             create_date=create_date,
@@ -541,15 +640,23 @@ class ScriptDirectory(object):
             message=message if message is not None else ("empty message"),
             **kw
         )
-        script = Script._from_path(self, path)
+
+        post_write_hooks = self.hook_config
+        if post_write_hooks:
+            write_hooks._run_hooks(path, post_write_hooks)
+
+        try:
+            script = Script._from_path(self, path)
+        except revision.RevisionError as err:
+            compat.raise_(util.CommandError(err.args[0]), from_=err)
         if branch_labels and not script.branch_labels:
             raise util.CommandError(
                 "Version %s specified branch_labels %s, however the "
                 "migration file %s does not have them; have you upgraded "
                 "your script.py.mako to include the "
-                "'branch_labels' section?" % (
-                    script.revision, branch_labels, script.path
-                ))
+                "'branch_labels' section?"
+                % (script.revision, branch_labels, script.path)
+            )
 
         self.revision_map.add_revision(script)
         return script
@@ -557,17 +664,18 @@ class ScriptDirectory(object):
     def _rev_path(self, path, rev_id, message, create_date):
         slug = "_".join(_slug_re.findall(message or "")).lower()
         if len(slug) > self.truncate_slug_length:
-            slug = slug[:self.truncate_slug_length].rsplit('_', 1)[0] + '_'
+            slug = slug[: self.truncate_slug_length].rsplit("_", 1)[0] + "_"
         filename = "%s.py" % (
-            self.file_template % {
-                'rev': rev_id,
-                'slug': slug,
-                'year': create_date.year,
-                'month': create_date.month,
-                'day': create_date.day,
-                'hour': create_date.hour,
-                'minute': create_date.minute,
-                'second': create_date.second
+            self.file_template
+            % {
+                "rev": rev_id,
+                "slug": slug,
+                "year": create_date.year,
+                "month": create_date.month,
+                "day": create_date.day,
+                "hour": create_date.hour,
+                "minute": create_date.minute,
+                "second": create_date.second,
             }
         )
         return os.path.join(path, filename)
@@ -589,9 +697,11 @@ class Script(revision.Revision):
             rev_id,
             module.down_revision,
             branch_labels=util.to_tuple(
-                getattr(module, 'branch_labels', None), default=()),
+                getattr(module, "branch_labels", None), default=()
+            ),
             dependencies=util.to_tuple(
-                getattr(module, 'depends_on', None), default=())
+                getattr(module, "depends_on", None), default=()
+            ),
         )
 
     module = None
@@ -600,6 +710,10 @@ class Script(revision.Revision):
     path = None
     """Filesystem path of the script."""
 
+    _db_current_indicator = None
+    """Utility variable which when set will cause string output to indicate
+    this is a "current" version in some database"""
+
     @property
     def doc(self):
         """Return the docstring given in the script."""
@@ -620,36 +734,37 @@ class Script(revision.Revision):
 
     @property
     def log_entry(self):
-        entry = "Rev: %s%s%s%s\n" % (
+        entry = "Rev: %s%s%s%s%s\n" % (
             self.revision,
             " (head)" if self.is_head else "",
             " (branchpoint)" if self.is_branch_point else "",
             " (mergepoint)" if self.is_merge_point else "",
+            " (current)" if self._db_current_indicator else "",
         )
         if self.is_merge_point:
-            entry += "Merges: %s\n" % (self._format_down_revision(), )
+            entry += "Merges: %s\n" % (self._format_down_revision(),)
         else:
-            entry += "Parent: %s\n" % (self._format_down_revision(), )
+            entry += "Parent: %s\n" % (self._format_down_revision(),)
 
         if self.dependencies:
             entry += "Also depends on: %s\n" % (
-                util.format_as_comma(self.dependencies))
+                util.format_as_comma(self.dependencies)
+            )
 
         if self.is_branch_point:
             entry += "Branches into: %s\n" % (
-                util.format_as_comma(self.nextrev))
+                util.format_as_comma(self.nextrev)
+            )
 
         if self.branch_labels:
             entry += "Branch names: %s\n" % (
-                util.format_as_comma(self.branch_labels), )
+                util.format_as_comma(self.branch_labels),
+            )
 
         entry += "Path: %s\n" % (self.path,)
 
         entry += "\n%s\n" % (
-            "\n".join(
-                "    %s" % para
-                for para in self.longdoc.splitlines()
-            )
+            "\n".join("    %s" % para for para in self.longdoc.splitlines())
         )
         return entry
 
@@ -660,35 +775,41 @@ class Script(revision.Revision):
             " (head)" if self.is_head else "",
             " (branchpoint)" if self.is_branch_point else "",
             " (mergepoint)" if self.is_merge_point else "",
-            self.doc)
+            self.doc,
+        )
 
     def _head_only(
-            self, include_branches=False, include_doc=False,
-            include_parents=False, tree_indicators=True,
-            head_indicators=True):
+        self,
+        include_branches=False,
+        include_doc=False,
+        include_parents=False,
+        tree_indicators=True,
+        head_indicators=True,
+    ):
         text = self.revision
         if include_parents:
             if self.dependencies:
                 text = "%s (%s) -> %s" % (
                     self._format_down_revision(),
                     util.format_as_comma(self.dependencies),
-                    text
+                    text,
                 )
             else:
-                text = "%s -> %s" % (
-                    self._format_down_revision(), text)
+                text = "%s -> %s" % (self._format_down_revision(), text)
         if include_branches and self.branch_labels:
             text += " (%s)" % util.format_as_comma(self.branch_labels)
         if head_indicators or tree_indicators:
-            text += "%s%s" % (
+            text += "%s%s%s" % (
                 " (head)" if self._is_real_head else "",
-                " (effective head)" if self.is_head and
-                    not self._is_real_head else ""
+                " (effective head)"
+                if self.is_head and not self._is_real_head
+                else "",
+                " (current)" if self._db_current_indicator else "",
             )
         if tree_indicators:
             text += "%s%s" % (
                 " (branchpoint)" if self.is_branch_point else "",
-                " (mergepoint)" if self.is_merge_point else ""
+                " (mergepoint)" if self.is_merge_point else "",
             )
         if include_doc:
             text += ", %s" % self.doc
@@ -696,15 +817,18 @@ class Script(revision.Revision):
 
     def cmd_format(
         self,
-            verbose,
-            include_branches=False, include_doc=False,
-            include_parents=False, tree_indicators=True):
+        verbose,
+        include_branches=False,
+        include_doc=False,
+        include_parents=False,
+        tree_indicators=True,
+    ):
         if verbose:
             return self.log_entry
         else:
             return self._head_only(
-                include_branches, include_doc,
-                include_parents, tree_indicators)
+                include_branches, include_doc, include_parents, tree_indicators
+            )
 
     def _format_down_revision(self):
         if not self.down_revision:
@@ -717,6 +841,29 @@ class Script(revision.Revision):
         dir_, filename = os.path.split(path)
         return cls._from_filename(scriptdir, dir_, filename)
 
+    @classmethod
+    def _list_py_dir(cls, scriptdir, path):
+        if scriptdir.sourceless:
+            # read files in version path, e.g. pyc or pyo files
+            # in the immediate path
+            paths = os.listdir(path)
+
+            names = set(fname.split(".")[0] for fname in paths)
+
+            # look for __pycache__
+            if os.path.exists(os.path.join(path, "__pycache__")):
+                # add all files from __pycache__ whose filename is not
+                # already in the names we got from the version directory.
+                # add as relative paths including __pycache__ token
+                paths.extend(
+                    os.path.join("__pycache__", pyc)
+                    for pyc in os.listdir(os.path.join(path, "__pycache__"))
+                    if pyc.split(".")[0] not in names
+                )
+            return paths
+        else:
+            return os.listdir(path)
+
     @classmethod
     def _from_filename(cls, scriptdir, dir_, filename):
         if scriptdir.sourceless:
@@ -730,8 +877,8 @@ class Script(revision.Revision):
         py_filename = py_match.group(1)
 
         if scriptdir.sourceless:
-            is_c = py_match.group(2) == 'c'
-            is_o = py_match.group(2) == 'o'
+            is_c = py_match.group(2) == "c"
+            is_o = py_match.group(2) == "o"
         else:
             is_c = is_o = False
 
@@ -757,7 +904,8 @@ class Script(revision.Revision):
                     "Be sure the 'revision' variable is "
                     "declared inside the script (please see 'Upgrading "
                     "from Alembic 0.1 to 0.2' in the documentation)."
-                    % filename)
+                    % filename
+                )
             else:
                 revision = m.group(1)
         else:
