@@ -8,6 +8,7 @@ import re
 from sqlalchemy import exc as sqla_exc
 from sqlalchemy import text
 
+from alembic import __version__
 from alembic import command
 from alembic import config
 from alembic import testing
@@ -16,6 +17,8 @@ from alembic.script import ScriptDirectory
 from alembic.testing import assert_raises
 from alembic.testing import assert_raises_message
 from alembic.testing import eq_
+from alembic.testing import is_false
+from alembic.testing import is_true
 from alembic.testing import mock
 from alembic.testing.env import _get_staging_directory
 from alembic.testing.env import _no_sql_testing_config
@@ -30,6 +33,8 @@ from alembic.testing.env import write_script
 from alembic.testing.fixtures import capture_context_buffer
 from alembic.testing.fixtures import capture_engine_context_buffer
 from alembic.testing.fixtures import TestBase
+from alembic.util import compat
+from alembic.util.sqla_compat import _connectable_has_table
 
 
 class _BufMixin(object):
@@ -198,6 +203,7 @@ finally:
 class CurrentTest(_BufMixin, TestBase):
     @classmethod
     def setup_class(cls):
+        cls.bind = _sqlite_file_db()
         cls.env = env = staging_env()
         cls.cfg = _sqlite_testing_config()
         cls.a1 = env.generate_revision("a1", "a1")
@@ -229,6 +235,12 @@ class CurrentTest(_BufMixin, TestBase):
 
         eq_(lines, set(revs))
 
+    def test_doesnt_create_alembic_version(self):
+        command.current(self.cfg)
+        engine = self.bind
+        with engine.connect() as conn:
+            is_false(_connectable_has_table(conn, "alembic_version", None))
+
     def test_no_current(self):
         command.stamp(self.cfg, ())
         with self._assert_lines([]):
@@ -387,7 +399,7 @@ finally:
         r2 = command.revision(self.cfg)
         db = _sqlite_file_db()
         command.upgrade(self.cfg, "head")
-        with db.connect() as conn:
+        with db.begin() as conn:
             conn.execute(
                 text("insert into alembic_version values ('%s')" % r2.revision)
             )
@@ -669,7 +681,7 @@ class StampMultipleHeadsTest(TestBase, _StampTest):
         command.stamp(self.cfg, [self.a])
 
         eng = _sqlite_file_db()
-        with eng.connect() as conn:
+        with eng.begin() as conn:
             result = conn.execute(
                 text("update alembic_version set version_num='fake'")
             )
@@ -1165,3 +1177,17 @@ class CommandLineTest(TestBase):
                     mock.call().close(),
                 ],
             )
+
+    def test_version_text(self):
+        buf = compat.StringIO()
+        to_mock = "sys.stdout" if util.compat.py3k else "sys.stderr"
+
+        with mock.patch(to_mock, buf):
+            try:
+                config.CommandLine(prog="test_prog").main(argv=["--version"])
+                assert False
+            except SystemExit:
+                pass
+
+        is_true("test_prog" in str(buf.getvalue()))
+        is_true(__version__ in str(buf.getvalue()))
